/*
 * (c) Copyright 2003 M. van Meegen
 * All Rights Reserved.
 * Sourcecode is licensed under Mozilla Public License 1.1
 */
package edu.virginia.cs.escjava;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Status;

/**
 * manages checkstyle configuration information
 * and caching of configured checkers.<p>
 * 
 * Instances of checker configuration are created internally for each
 * project for which a checker is requested.<p>
 * 
 * Checkers and configurations are cached in a static configuration cache
 * mapping project name to EscjavaConfiguration.
 * 
 * The EscjavaConfiguration to use for a file is retrieved hierarchically
 * by searching the following places until a property is found:
 * <ol>
 * <li>project configuration
 * <li>checkstyle preferences
 * <li>Default settings including configuration files in plugin dir
 * </ol>
 * 
 * @author Marco van Meegen
 */
public class EscjavaConfiguration {

	/** variable substituted in properties */
	private static final String VAR_SAMEDIR = "$SAMEDIR";
	
	/** location of esc/java executable */
	public static final String P_ESCJAVA_FILE = "filenames.escjava";

	/** filter file name property */
	public static final String P_FILTER_FILE = "filenames.filter";

	/** configuration file name property */
	public static final String P_CONFIG_FILE = "filenames.config";

	/** package name file name property */
	public static final String P_PACKAGENAMES_FILE = "filenames.packagenames";

	/** properties file name property */
	public static final String P_PROPERTIES_FILE = "filenames.properties";

	/** enabled-state (boolean) property */
	public static final String P_ENABLED = "false";

	/** apply automatic rebuild on configuration changes */
	public static final String P_AUTO_REBUILD = "true";

	/** map containing default property values */
	private static final Map DEFAULTS;

	static {
		// init defaults map
		DEFAULTS = new HashMap();
		// DEFAULTS.put(P_ESCJAVA_FILE, "c:\\Program Files\\java\\escjava\\bin\\escjava.bat");
		DEFAULTS.put(P_ESCJAVA_FILE, "C:/Program Files/Java/escjava/bin/escjava.bat");
		DEFAULTS.put(P_FILTER_FILE, "resources/nofilters.flt");
		DEFAULTS.put(P_CONFIG_FILE, "resources/default_checks.xml");
		DEFAULTS.put(P_PACKAGENAMES_FILE, "resources/checkstyle_packages.xml");
		DEFAULTS.put(P_PROPERTIES_FILE, "resources/toplevel.properties");
	}

	// --------------------------------------------------------------------
	//
	// publicly accessible static methods which represent cache management
	//
	// --------------------------------------------------------------------

	/** gets a preconfigured checker for the given project 
	 * @param project project to create checker for
	 * @return checker from cache or newly created, configured for the given project
	 * @throws IOException checker could not be properly configured; probably one of
	 *  the given configuration files did not exist
	 */
	public static Checker getChecker(IProject project) throws IOException {
		EscjavaConfiguration configurationForProject = getEscjavaConfiguration(project);

		return configurationForProject.getChecker();
	}

	/** gets (or creates) the configuration related to the project
	 * @param project project to retrieve configuration for
	 * @return configuration for this project
	 */
	public static EscjavaConfiguration getEscjavaConfiguration(IProject project) {
		// look up configuration for project in cache
		EscjavaConfiguration configurationForProject = (EscjavaConfiguration) configurationCache.get(project.getName());
		
		if (configurationForProject == null) {
			// none found ? create a new one and put it in map			
			configurationForProject = new EscjavaConfiguration(project);
			configurationCache.put(project.getName(), configurationForProject);
		}
		return configurationForProject;
	}

	/** gets a preconfigured global checker 
	 * @return newly created checker configured for the global preferences.
	 *  Checker must be destroyed by client !
	 * @throws IOException checker could not be properly configured; probably one of
	 *  the given configuration files did not exist
	 */
	public static Checker getGlobalChecker() throws IOException {
		EscjavaConfiguration configuration = new EscjavaConfiguration(null);
		return configuration.getChecker();
	}

	/** retrieves the string property for the given project.
	 * All hierarchy levels are searched until one is found.
	 * @param propertyName property to get
	 * @param project project to start search
	 * @return property value, if found somewhere on the hierarchy levels;
	 *   otherwise null
	 */
	public static String getStringProperty(String propertyName, IProject project) {
		// search different levels to find property
		String result = null;

		// first level: project
		if (project != null) {
			EscjavaPreferenceStore prefs = new EscjavaPreferenceStore(project);
			result = prefs.getString(propertyName);
			if (result.length() == 0) {
				result = null;
			}
		}

		// second level: preferences
		Preferences prefs = EscjavaPlugin.getDefault().getPluginPreferences();
		if (result == null) {
			result = prefs.getString(propertyName);
			if (result.length() == 0) {
				result = null;
			}
		}

		// third level: default
		if (result == null) {
			result = (String) DEFAULTS.get(propertyName);
		}

		return (String) result;
	}

	/** retrieves the string property for the given project.
	 * All hierarchy levels are searched until one is found.
	 * @param propertyName property to get
	 * @param project project to start search
	 * @return fully qualified file name if filename is specified somewhere in the hierarchy,
	 *   otherwise null
	 */
	public static String getFileNameForProperty(String propertyName, IProject project) {
		// search different levels to find property
		String result = null;

		String name = null;
		// first level: project
		try {
			if (project != null) {
				EscjavaPreferenceStore prefs = new EscjavaPreferenceStore(project);
				name = prefs.getString(propertyName);
			}
			if (name != null && name.length() > 0) {
				// convert
				result = project.getWorkspace().getRoot().getFile(new Path(name)).getLocation().toOSString();
			}

		} catch (Exception e) {
			// new try as absolute filename
			result = name;
		}

		// second level: preferences
		if (result == null) {
			Preferences prefs = EscjavaPlugin.getDefault().getPluginPreferences();
			result = prefs.getString(propertyName);
			// not defined returns empty string, so set this to null
			if (result.length() == 0) {
				result = null;
			}
		}

		// third level: default name in plugin-dir
		if (result == null) {
			try {
				name = (String) DEFAULTS.get(propertyName);
				result = getFilePath(name);

			} catch (Exception e) {
				EscjavaPlugin.log("Could not get default filename property", e);
				result = null;
			}
		}

		return (String) result;
	}

	/** retrieves the boolean property for the given project.
	 * All hierarchy levels are searched until one is found.
	 * @param propertyName property to get
	 * @param project project to start search
	 * @return property value, if found somewhere on the hierarchy levels;
	 *   otherwise false
	 */
	public static boolean getBooleanProperty(String propertyName, IProject project) {
		// search only project to find property
		boolean result = false;

		try {
			if (project != null) {
				EscjavaPreferenceStore prefs = new EscjavaPreferenceStore(project);
				result = prefs.getBoolean(propertyName);
			}
		} catch (Exception e) {
			EscjavaPlugin.log("Could not get default boolean property", e);
			result = false;
		}
		return result;
	}

	/** invalidates cached configuration information */
	public static void invalidate() {
		for (Iterator it = configurationCache.values().iterator(); it.hasNext();) {
			EscjavaConfiguration config = (EscjavaConfiguration) it.next();
			config.destroy();
		}
		configurationCache.clear();
	}

	/** invalidates cached configuration information 
	 * @param project project to invalidate information for
	 */
	public static void invalidate(IProject project) {
		// look up configuration for project in cache
		EscjavaConfiguration configurationForProject = (EscjavaConfiguration) configurationCache.get(project.getName());

		// found, destroy configuration
		if (configurationForProject != null) {
			configurationCache.remove(project.getName());
			configurationForProject.destroy();
		}
	}

	// --------------------------------------------------------------------
	//
	// internal methods for configuration management
	//
	// --------------------------------------------------------------------

	/**
	 * destroys checker, removes listeners and deletes all stuff
	 * for the maintained checker
	 */
	private void destroy() {
		if (checker != null) {
			checker.destroy();
			checker = null;
		}
	}

	/**
	 * @return checker preconfigured for this configuration
	 * @throws IOException one of the configuration files could not be read
	 */
	private Checker getChecker() throws IOException {
		// checker exists ?
		if (checker == null) {
			// no, create
			checker = createChecker();
		}

		return checker;
	}

	/**
	 * creates a new configuration for the given project
	 * @param project project to search for configuration information; 
	 * 	null: no project is used for config information
	 */
	private EscjavaConfiguration(IProject project) {
		super();
		this.project = project;
	}

	/**
	 * 
	 * @return checker configured with the configuration information for this
	 * @throws IOException IOException if some config file could not be loaded
	 */
	private Checker createChecker() throws IOException {
		boolean errors = false;
		String projectDir = null;
		if (project != null) {
			projectDir = project.getLocation().toOSString();
			EscjavaPlugin.log("creating checker for Project: " + project.getName(), Status.INFO);
		} else {
			EscjavaPlugin.log("creating global checker", Status.INFO);
		}
		
		// load properties from Project config
		Properties properties = new Properties();
		String propertiesFileName = getPropertiesFileName();
		if (propertiesFileName != null) {
			EscjavaPlugin.log("Properties file: " + propertiesFileName, Status.INFO);
			FileInputStream fis = null;
			try {
				fis = new FileInputStream(propertiesFileName);
				properties.load(fis);
				
				// hack: and patch $SAMEDIR to enable file paths relative to properties dir
				int dirSep = propertiesFileName.lastIndexOf('/');
				if (dirSep == -1) {
					dirSep = propertiesFileName.lastIndexOf('\\');
				}
				String propertiesDir;
				if (dirSep != -1) {
					propertiesDir = propertiesFileName.substring(0, dirSep);
				 } else {
					propertiesDir = ".";
				}
				for (Iterator it = properties.entrySet().iterator(); it.hasNext();) {
					Map.Entry entry = (Map.Entry) it.next();
					String element = (String) entry.getValue();
					int i = element.indexOf(VAR_SAMEDIR);
					if (i >= 0) {
						String substituted = element.substring(0, i) + propertiesDir + element.substring(i + VAR_SAMEDIR.length());
						entry.setValue(substituted);
					}
						
					
				}
				
			} catch (IOException ex) {
				errors = true;
				EscjavaPlugin.log("Unable to load properties from file: " + propertiesFileName, ex);
				throw new IOException("could not load properties file " + propertiesFileName 
				   + "\ncause: " + ex.getLocalizedMessage());
			} finally {
				if (fis != null) {
					fis.close();
				}
			}
		}

		// create the checker
		Checker c = null;
		try {
			c = new Checker();
			c.setBasedir(projectDir);
		} catch (Throwable ex) {
			errors = true;
			EscjavaPlugin.log("Unable to create Checker", ex);
			throw new IOException("could not create Checker " 
			   + "\ncause: " + ex.getLocalizedMessage());
		}


		// for future bundled error handling
		if (errors) {
			EscjavaPlugin.log(
				"Errors occured creating "
					+ (project != null ? "checker for project " + project.getName() : "global checker"), Status.ERROR);
			EscjavaPlugin.log("Double Check configuration parameters, syntax and file names !!!", Status.ERROR);
			throw new IOException("Error creating checker. Double Check configuration parameters, syntax and file names !!!");
		} else {
			EscjavaPlugin.log("Checker successfully created and configured.", Status.INFO);
		}
		
		return c;
	}

	private String getEscjavaFileName() {
		return getFileNameForProperty(P_ESCJAVA_FILE, project);
	}
	
	/** @return an absolute file path for the given plugin-directory-relative filepath 
	 * @param relativePath path relative to plugin-dir, e.g. resources/checks.xml
	 * @throws IOException if path not exists
	 */
	private static String getFilePath(String relativePath) throws IOException {
		URL toolUrl = EscjavaPlugin.getDefault().find(new Path(relativePath));
		if (toolUrl == null) {
			throw new IOException("File not found in plugin dir " + relativePath);
		}
		return toolUrl != null ? toolUrl.getFile() : null;
	}

	/** @return the package name file name to use */
	private String getPackageNameFileName() {
		return getFileNameForProperty(P_PACKAGENAMES_FILE, project);
	}

	/**
	 * @return the checkstyle configuration file name to use
	 */
	private String getConfigFileName() {
		return getFileNameForProperty(P_CONFIG_FILE, project);
	}

	/**
	 * @return the checkstyle properties file name to use
	 */
	private String getPropertiesFileName() {
		return getFileNameForProperty(P_PROPERTIES_FILE, project);
	}

	/**
	 * @return the filter file name to use
	 */
	private String getFilterFileName() {
		return getFileNameForProperty(P_FILTER_FILE, project);
	}

	/** checker for this configuration */
	private Checker checker = null;

	/** project this configuration is associated with */
	private IProject project;

	/** cache of checker configurations */
	private static Map configurationCache = new HashMap(10);
	
	/** filter list for this project */
	private List fileFilters = null;

}
