////////////////////////////////////////////////////////////////////////////////
// checkstyle: Checks Java source code for adherence to a set of rules.
// Copyright (C) 2001-2002  Oliver Burn
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package edu.virginia.cs.escjava;

import java.util.Vector;


/**
 * Represents a message that can be localised. The translations come from
 * message.properties files. The underlying implementation uses
 * java.text.MessageFormat.
 *
 * @author <a href="mailto:checkstyle@puppycrawl.com">Oliver Burn</a>
 * @author lkuehne
 * @version 1.0
 */
public final class EscjavaMessage
{

    /** the line number **/
    private final int mLineNo;
    /** the column number **/
    private final int mColNo;

    /** the severity level **/
    private final SeverityLevel mSeverityLevel;

    /** the default severity level if one is not specified */
    private static final SeverityLevel DEFAULT_SEVERITY = SeverityLevel.ERROR;

	/** description of ESC/Java message */
	private final String mDescription;
	
	/** source code filename that message came from */
	private final String mSourceName;

	private String mText;
	
	private Vector detailedMessage; // Further lines of a message
	
    /**
     * Creates a new <code>LocalizedMessage</code> instance.
     *
     * @param aLineNo line number associated with the message
     * @param aColNo column number associated with the message
     * @param aBundle resource bundle name
     * @param aKey the key to locate the translation
     * @param aArgs arguments for the translation
     * @param aSeverityLevel severity level for the message
     * @param aSourceName the name of the source of the message
     */
    public EscjavaMessage(int aLineNo,
                            int aColNo,
							String aDescription,
                            SeverityLevel aSeverityLevel,
                            String aSourceName)
    {
        mLineNo = aLineNo;
        mColNo = aColNo;
        mDescription = aDescription;
        mSeverityLevel = aSeverityLevel;
        mSourceName = aSourceName;
        mText = "";
        detailedMessage = new Vector();
    }

    public Vector getDetailedMessage()
    {
    	return detailedMessage;
    }
    
    public void setText(String aText)
    {
    	mText = aText;
    }
    
    public void addMessage(String message)
    {
    	detailedMessage.add(message);
    }
    
    /** @return the translated message **/
    public String getDescription()
    {
    	return mDescription;
    }

	public String getText()
	{
		return mText;
	}

    /** @return the line number **/
    public int getLineNo()
    {
        return mLineNo;
    }

    /** @return the column number **/
    public int getColumnNo()
    {
        return mColNo;
    }

    /** @return the severity level **/
    public SeverityLevel getSeverityLevel()
    {
        return mSeverityLevel;
    }

    /** @return the name of the source for this LocalizedMessage */
    public String getSourceName()
    {
        return mSourceName;
    }
    
    public Object clone()
    {
    	EscjavaMessage ret = new EscjavaMessage(getLineNo(),
    								getColumnNo(),
    								getDescription(),
    								getSeverityLevel(),
    								getSourceName());
    	return ret;
    }
}
